/*
 Copyright (c) 2013 aizo ag, Zurich, Switzerland

 Author: Christain Hitz <christian.hitz@aizo.com>

 This file is part of dS Assistant.

 dS-Assistant is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 dS-Assistant is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

 NOTE: all APIs, especially the JSON API, offered by this application
 are subject to change without further notice.
 */

#ifndef __DEVICE_CONFIG_H__
#define __DEVICE_CONFIG_H__

#include <boost/shared_ptr.hpp>
#include <digitalSTROM/ds.h>
#include <inttypes.h>
#include <string>
#include <vector>
#include "hwdesc.h"

/// \file device-config.h
/// \brief

namespace dSA {

struct ParameterSpec
{
    uint8_t bank;
    uint8_t offset;
    uint8_t value;
    uint8_t mask;
};

class SceneConfig
{
public:
    SceneConfig(int dimmTime, int ledConfig, bool flash, bool special,
                bool localprio, bool dontcare);
    virtual ~SceneConfig() {};

    virtual bool isEqual(const boost::shared_ptr<SceneConfig>& config) const;
    virtual boost::shared_ptr<ParameterSpec> getParameterSpec() const;

    int getDimmTime() const { return m_dimmTime; }
    void setDimmTime(int dimmTime) { m_dimmTime = dimmTime; }
    bool isDontcare() const { return m_dontcare; }
    void setDontcare(bool dontcare) { m_dontcare = dontcare; }
    bool isFlash() const { return m_flash; }
    void setFlash(bool flash) { m_flash = flash; }
    int getLedConfig() const { return m_ledConfig; }
    void setLedConfig(int ledConfig) {  m_ledConfig = ledConfig; }
    bool isLocalprio() const { return m_localprio; }
    void setLocalprio(bool localprio) { m_localprio = localprio; }
    bool isSpecial() const { return m_special; }
    void setSpecial(bool special) { m_special = special; }

protected:
    int m_dimmTime;
    int m_ledConfig;
    bool m_flash;
    bool m_special;
    bool m_localprio;
    bool m_dontcare;
};

class Scene
{
public:
    Scene(int sceneNumber, bool active, boost::shared_ptr<SceneConfig>& sceneConfig);
    virtual ~Scene() {};

    virtual int getValue() const = 0;
    virtual bool isEqual(const Scene* scene) const = 0;
    virtual std::vector<boost::shared_ptr<ParameterSpec> > parameterChanges(
            const Scene* scene) const = 0;
    int getSceneNumber() const { return m_sceneNumber; }
    void setSceneNumber(int sceneNumber) { m_sceneNumber = sceneNumber; }
    const boost::shared_ptr<SceneConfig>& getSceneConfig() const
            { return m_sceneConfig; }
    void setSceneConfig(const boost::shared_ptr<SceneConfig>& sceneConfig)
            { m_sceneConfig = sceneConfig; }
    bool isActive() const { return m_active; }
    void setActive(bool active) { m_active = active; }

protected:
    std::vector<boost::shared_ptr<ParameterSpec> > parameterChangesConfig(
            const Scene* scene) const;
    virtual bool isConfigEqual(const Scene* scene) const;
    virtual std::vector<boost::shared_ptr<ParameterSpec> > getParameterSpec() const;

protected:
    int m_sceneNumber;
    bool m_active;
    boost::shared_ptr<SceneConfig> m_sceneConfig;
};

class LightScene : public Scene
{
public:
    LightScene(int sceneNumber, bool active, uint8_t value,
               boost::shared_ptr<SceneConfig>& sceneConfig);
    virtual ~LightScene() {};
    virtual bool isEqual(const Scene* config) const;
    virtual std::vector<boost::shared_ptr<ParameterSpec> > parameterChanges(
            const Scene* scene) const;

    int getValue() const { return m_value; }
    void setValue(uint8_t value) { m_value = value; }

protected:
    virtual std::vector<boost::shared_ptr<ParameterSpec> > getParameterSpec() const;

protected:
    uint8_t m_value;
};

class MotorScene : public Scene
{
public:
    MotorScene(int sceneNumber, bool active, uint16_t value,
               boost::shared_ptr<SceneConfig>& sceneConfig);
    virtual ~MotorScene() {};
    virtual bool isEqual(const Scene* scene) const;
    virtual std::vector<boost::shared_ptr<ParameterSpec> > parameterChanges(
            const Scene* scene) const;

    int getValue() const { return m_value; }
    void setValue(uint16_t value) { m_value = value; }

protected:
    virtual std::vector<boost::shared_ptr<ParameterSpec> > getParameterSpec() const;

protected:
    uint16_t m_value;
};

class JalusieScene : public MotorScene
{
public:
    JalusieScene(int sceneNumber, bool active, uint16_t value, uint8_t angle,
                 boost::shared_ptr<SceneConfig>& sceneConfig);
    virtual ~JalusieScene() {};
    virtual bool isEqual(const Scene* config) const;
    virtual std::vector<boost::shared_ptr<ParameterSpec> > parameterChanges(
            const Scene* scene) const;
    virtual std::vector<boost::shared_ptr<ParameterSpec> > getParameterSpec() const;

    uint8_t getAngle() const { return m_angle; }
    void setAngle(uint8_t angle) { m_angle = angle; }

protected:
    uint8_t m_angle;
};

class DeviceConfig
{
public:
    DeviceConfig(uint8_t colorId, uint16_t productId, uint16_t fwVersion,
                 const std::string& name, int version);
    ~DeviceConfig() {};

    void addScene(boost::shared_ptr<Scene> scene);
    const std::vector<boost::shared_ptr<Scene> >& getScenes() const
            { return m_vScene; }
    void addParameter(boost::shared_ptr<ParameterSpec> parameter);
    const std::vector<boost::shared_ptr<ParameterSpec> >& getParameters() const
            { return m_vParameters; }
    std::vector<boost::shared_ptr<ParameterSpec> > parameterChanges(
            const DeviceConfig& other);
    std::vector<int> sceneChanges(const DeviceConfig& other);

    int getVersion() const { return m_version; }
    void setVersion(int version) { this->m_version = version; }
    uint8_t getColorId() const { return m_colorId; }
    void setColorId(uint8_t colorId) { m_colorId = colorId; }
    uint16_t getProductId() const { return m_productId; }
    void setProductId(uint16_t productId) { m_productId = productId; }
    const std::string& getName() const { return m_name; }
    void setName(const std::string& name) { m_name = name; }
    uint16_t getFwVersion() const { return m_fwVersion; }
    void setFwVersion(uint16_t fwVersion) { m_fwVersion = fwVersion; }

private:
    int m_version;
    uint8_t m_colorId;
    uint16_t m_productId;
    uint16_t m_fwVersion;
    std::string m_name;
    std::vector<boost::shared_ptr<Scene> > m_vScene;
    std::vector<boost::shared_ptr<ParameterSpec> > m_vParameters;
};

class DeviceConfigReader
{
public:
    typedef enum {
        LIGHT_SCENE = 0,
        MOTOR_SCENE = 1,
        JALUSIE_SCENE = 2,
    } SceneType;

    boost::shared_ptr<DeviceConfig> read(const Device& device,
                                         int version);

private:
    std::string get_file_contents(const std::string& filename);
};

};

#endif//__DEVICE_CONFIG_H__
